; Copyright (C) 1995-1997 The Hungry Programmers - hungry@hungry.com
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; from /usr/X11/include/X11/X.h
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define xt-no-event-mask 0)
(define xt-key-press-mask 1)
(define xt-key-release-mask 2)
(define xt-button-press-mask 4)
(define xt-button-release-mask 8)
(define xt-enter-window-mask 16)
(define xt-leave-window-mask 32)
(define xt-pointer-motion-mask 64)
(define xt-pointer-motion-hint-mask 128)
(define xt-button1-motion-mask 256)
(define xt-button2-motion-mask 512)
(define xt-button3-motion-mask 1024)
(define xt-button4-motion-mask 2048)
(define xt-button5-motion-mask 4096)
(define xt-button-motion-mask 8192)
(define xt-keymap-state-mask 16384)
(define xt-exposure-mask 32768)
(define xt-visibility-change-mask 65536)
(define xt-structure-notify-mask 131072)
(define xt-resize-redirect-mask 262144)
(define xt-substructure-notify-mask 524288)
(define xt-substructure-redirect-mask 262144)
(define xt-focus-change-mask 2097152)
(define xt-property-change-mask 4194304)
(define xt-colormap-change-mask 8388608)
(define xt-owner-grab-button-mask 16777216)

(define xt-key-press 2)
(define xt-key-release 3)
(define xt-button-press 4)
(define xt-button-release 5)
(define xt-motion-notify 6)
(define xt-enter-notify 7)
(define xt-leave-notify 8)
(define xt-focus-in 9)
(define xt-focus-out 10)
(define xt-keymap-notify 11)
(define xt-expose 12)
(define xt-graphics-expose 13)
(define xt-no-expose 14)
(define xt-visibility-notify 15)
(define xt-create-notify 16)
(define xt-destroy-notify 17)
(define xt-unmap-notify 18)
(define xt-map-notify 19)
(define xt-map-request 20)
(define xt-reparent-notify 21)
(define xt-configure-notify 22)
(define xt-configure-request 23)
(define xt-gravity-notify 24)
(define xt-resize-request 25)
(define xt-circulate-notify 26)
(define xt-circulate-request 27)
(define xt-property-notify 28)
(define xt-selection-clear 29)
(define xt-selection-request 30)
(define xt-selection-notify 31)
(define xt-colormap-notify 32)
(define xt-client-message 33)
(define xt-mapping-notify 34)


(define (form-menu menu-bar title items)
  (let* ((pulldown (xt-create-pulldown-menu menu-bar))
	 (cascade (xt-create-widget title
				    (xm-cascade-button-widget-class)
				    menu-bar
				    #f))
	 (title-str (xm-string-create-localized title)))
    (xt-set-value cascade (xm-n-sub-menu-id) pulldown)
    (xt-set-value cascade (xm-n-label-string) title-str)
    (xm-string-free title-str)
    (xt-manage-child cascade)
  
    (for-each (lambda (m)
		(let ((bp (xt-create-widget (car m)
					    (xm-push-button-widget-class)
					    pulldown
					    #f)))
		  (xt-manage-child bp)
		  (xt-add-callback bp (xm-n-activate-callback)
				   (cadr m) (cddr m))))
	      items)
    cascade))


(define (add-combo-box-item combo-box item)
  (let* ((xmstr (xm-string-create-localized item)))
    (xm-combo-box-add-item combo-box xmstr)
    (xm-string-free xmstr)))



(define (load-pixmap filename)
  (car (xdisplay.read-ppm-file-to-pixmap xd (find-the-file filename))))


(define (form-tool-bar parent keys-to buttons)
  (let* ((tool-frame (xt-create-widget
		      "sub-tool-frame"
		      (xm-frame-widget-class)
		      parent
		      #f))
	 (tool-form (xt-create-widget
		     "sub-tool-form"
		     (xm-form-widget-class)
		     tool-frame
		     #f)))
    (do ((b buttons (cdr b))
	 (leftwidget #f))
	((null? b) #t)
      (let* ((x (car b))
	     (ppm (car x))
	     (px (load-pixmap ppm))
	     (callback (cadr x))
	     (args (caddr x))
	     (button (xt-create-widget
		      "tool-bar-button"
		      (xm-push-button-widget-class)
		      tool-form
		      (list
		       (list (xm-n-label-type) (xm-pixmap))
		       (list (xm-n-label-pixmap) px)
		       ;(list (xm-n-label-insensitive-pixmap) ipx)
		       (list (xm-n-top-attachment) (xm-attach-form))
		       (list (xm-n-right-attachment) (xm-attach-none))
		       (list (xm-n-bottom-attachment) (xm-attach-form))))))
	(if (eqv? leftwidget #f)
	    (xt-set-value button (xm-n-left-attachment) (xm-attach-form))
	    (begin
	      (xt-set-value button (xm-n-left-attachment) (xm-attach-widget))
	      (xt-set-value button (xm-n-left-widget) leftwidget)))
	(xt-add-callback button (xm-n-activate-callback) callback args)
	(xt-manage-child button)
	(xt-set-keyboard-focus button keys-to)
	(set! leftwidget button)))
    (xt-manage-child tool-form)
    tool-frame))


(define (form-label parent str args)
  (let* ((xm-string (xm-string-create-localized str))
	 (label (xt-create-widget str
				  (xm-label-widget-class)
				  parent
				  (list-append
				   (list
				    (list (xm-n-label-string) xm-string))
				   args))))
    (xm-string-free xm-string)
    (xt-manage-child label)
    label))


(define (xm-text-set-string tf value)
  (xt-set-value tf (xm-n-value) 
		(scm-string-to-c-string
		 (if (number? value) (number->string value) value))))

(define (xm-text-get-string tf)
  (c-string-to-scm-string (xt-get-value tf (xm-n-value))))
