; Copyright (C) 1995-1997 The Hungry Programmers - hungry@hungry.com
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


(define landscape-pix #f)
(define portrait-pix #f)


; paper-types entries are:
; ("name" width height left-m top-m right-m bottom-m)
; the last 4 are default margins

(define paper-types
  (list
   (list "US Letter" 612 792 72 72 72 72) ; 8.5" x 11"
   (list "US Legal" 612 1008 72 72 72 72) ; 8.5" x 14"
   (list "A3" 843 1192 80 80 80 80)  ; 297mm x 420mm
   (list "A4" 596 843 80 80 80 80) ; 210mm x 297mm
   (list "A5" 422 596 80 80 80 80) ; 148mm x 210mm
   (list "B4" 730 1034 60 60 60 60) ; 257mm x 364mm
   (list "B5" 500 710 80 80 80 80) ; 176mm x 250mm
   (list "B5 Japan" 517 730 80 80 80 80) ; 182mm x 257mm
   (list "Half Letter" 612 397 72 72 72 72) ; 5.5" x 8.5"
   (list "Executive" 523 758 72 72 72 72) ; 8.25" x 10.5"
   (list "Tabloid/Ledger" 794 1227 72 72 72 72) ; 11" x 17"
   (list "Monarch" 280 542 10 10 10 10) ; 3.875" x 7.5"
   (list "SuperB" 843 1227 80 80 80 80) ; 297mm x 432mm
   (list "Envelope Commercial #10" 298 686 5 5 5 5) ; 4.125" x 9.5"
   (list "Envelope Monarch #7 3/4" 280 542 5 5 5 5) ; 3.875" x 7.5"
   (list "Envelope DL" 312 625 5 5 5 5) ; 110mm x 220mm
   (list "Envelope C5" 460 650 5 5 5 5) ; 162mm x 229mm
   (list "EuroPostcard " 298 420 5 5 5 5) ; 105mm x 148mm
   ))

; 72.2 points in an inch?
; 2.84 points in a mm?

;<Veep> "letter" is 8.5x11. "legal" is 8.5x14. "A4" has metric measuremetns
;<Veep> Sometimes PS files made for A4 look right on letter paper,
;       and sometimes not
;<piglet3> so page numbers on a4 documents get chopped off on us printers.

;  US Letter        8.5"   x  11.0"
;  Half Letter      5.5"   x   8.5"
;  US Legal         8.5"   x  14.0"
;  Statement
;  Executive        7.25"  x  10.5"  ; -- netscape says 7.5" x 10" ???
;  Tabloid/Ledger  11.0"   x  17.0"
;  Folio
;  A2
;  A3              297mm   x  420mm
;  A4              210mm   x  297mm
;  A5              148.5mm x  210mm
;  B3
;  B4
;  B5              182mm   x  257mm
;  B5(anotherpage) 178mm   x  250mm
;  SuperB          297mm   x  432mm
; Envelope:
;  Commercial #10   4.125" x   9.5"
;  Monarch #7 3/4   3.875" x   7.5"
;  DL              110mm   x  220mm
;  C5              162mm   x  229mm
;  Postcard
;  EuroPostcard    105mm   x  148mm

;;;
;        A0                      C
;        A1                      D
;        A2                      E
;        A3                      B4
;        A4                      B5
;        A5                      B6
;        7X9                     C4_ENVELOPE
;        10X14                   C5_ENVELOPE
;        11X14                   C6_ENVELOPE
;        LEGAL                   DL_ENVELOPE
;        EXECUTIVE               7_ENVELOPE
;        HALFLETTER              9X12_ENVELOPE
;        LETTER (same as A)      10X13_ENVELOPE
;        LEDGER (same as B)      TWOTHIRDSA4
;        FOLIO                   BUSINESS_ENVELOPE


; returns an index into paper-types
(define (current-paper-type)
  (let* ((page-width (attributes.get-i (car page-styles) at-fixed-right))
	 (page-height (attributes.get-i (car column-styles) at-fixed-down)))
    (do ((pt paper-types (cdr pt)))
	((or (null? pt)
	     (and (eqv? (cadr (car pt)) page-width)
		  (eqv? (caddr (car pt)) page-height))
	     (and (eqv? (cadr (car pt)) page-height)
		  (eqv? (caddr (car pt)) page-width)))
	 (cond ((null? pt) (list 0 'portrait))
	       ((and (eqv? (cadr (car pt)) page-width)
		     (eqv? (caddr (car pt)) page-height))
		(list (- (length paper-types) (length pt)) 'portrait))
	       ((and (eqv? (cadr (car pt)) page-height)
		     (eqv? (caddr (car pt)) page-width))
		(list (- (length paper-types) (length pt)) 'landscape)))))))


(define (paper-name-to-index str)
  (do ((pt paper-types (cdr pt)))
      ((or (null? pt) (string=? (car (car pt)) str))
       (if (null? pt) 0 (- (length paper-types) (length pt))))))



(define (set-page-size w h)
  
  (attributes.add-i (car page-styles) at-fixed-right (* magnify w))
  (attributes.add-i (car column-styles) at-fixed-down (* magnify h))
  (attributes.add-i (cadr column-styles) at-fixed-down (* magnify h))
  (attributes.add-i (cadddr column-styles) at-fixed-down (* magnify h))
  (attributes.add-i (caddr line-styles-low) at-fixed-down (* magnify h))
  (attributes.add-i (caddr line-styles-center) at-fixed-down (* magnify h))
  (attributes.add-i (caddr line-styles-high) at-fixed-down (* magnify h))
  (attributes.add-i (caddr line-styles-fill) at-fixed-down (* magnify h))
  (set-widths-by-page-width)
  )


(define (set-margins left top right bottom)
  (attributes.add-i (car page-styles) at-margin-low left)
  (attributes.add-i (car page-styles) at-margin-high right)
  (attributes.add-i (car column-styles) at-margin-low top)
  (attributes.add-i (car column-styles) at-margin-high bottom)
  (attributes.add-i (cadr column-styles) at-margin-low top)
  (attributes.add-i (cadr column-styles) at-margin-high bottom)
  ;(attributes.add-i (caddr column-styles) at-margin-low top) ; not for asian
  ;(attributes.add-i (caddr column-styles) at-margin-high bottom)
  (attributes.add-i (cadddr column-styles) at-margin-low top)
  (attributes.add-i (cadddr column-styles) at-margin-high bottom)

  (attributes.add-i (caddr line-styles-low) at-margin-low top)
  (attributes.add-i (caddr line-styles-low) at-margin-high bottom)
  (attributes.add-i (caddr line-styles-center) at-margin-low top)
  (attributes.add-i (caddr line-styles-center) at-margin-high bottom)
  (attributes.add-i (caddr line-styles-high) at-margin-low top)
  (attributes.add-i (caddr line-styles-high) at-margin-high bottom)
  (attributes.add-i (caddr line-styles-fill) at-margin-low top)
  (attributes.add-i (caddr line-styles-fill) at-margin-high bottom))


(define (popup-page-settings parent)

  (if (eqv? landscape-pix #f)
      (set! landscape-pix (load-pixmap "ppms/landscape.ppm")))
  (if (eqv? portrait-pix #f)
      (set! portrait-pix (load-pixmap "ppms/portrait.ppm")))

  (let* ((dlg (xm-create-message-dialog parent "Page Settings" #f))

	 (frame (xt-create-widget
		 "page-settings-frame"
		 (xm-frame-widget-class)
		 dlg #f))

	 (form (xt-create-widget "Form" (xm-form-widget-class) frame #f))

	 (rc-frame (xt-create-widget
		    "rc frame"
		    (xm-frame-widget-class) form
		    (list
		     (list (xm-n-left-attachment) (xm-attach-form))
		     (list (xm-n-top-attachment) (xm-attach-form))
		     (list (xm-n-right-attachment) (xm-attach-none))
		     (list (xm-n-bottom-attachment) (xm-attach-none)))))

	 (rc (xt-create-widget
	      "page-settings-rc" (xm-row-column-widget-class)
	      rc-frame (list (list (xm-n-num-columns) 2)
			     (list (xm-n-packing) (xm-pack-column)))))

	 (paper-combo (xt-create-widget
		       "Paper Combo"
		       (xm-combo-box-widget-class)
		       form
		       (list
			(list (xm-n-left-attachment) (xm-attach-widget))
			(list (xm-n-left-widget) rc)
			(list (xm-n-top-attachment) (xm-attach-form))
			(list (xm-n-right-attachment) (xm-attach-form))
			(list (xm-n-bottom-attachment) (xm-attach-none)))))

	 (layout-combo (xt-create-widget
			"Layout Combo"
			(xm-combo-box-widget-class)
			form
			(list
			 (list (xm-n-left-attachment) (xm-attach-widget))
			 (list (xm-n-left-widget) rc)
			 (list (xm-n-top-attachment) (xm-attach-widget))
			 (list (xm-n-top-widget) paper-combo)
			 (list (xm-n-right-attachment) (xm-attach-form))
			 (list (xm-n-bottom-attachment) (xm-attach-none)))))

	 (set-def-marg (xt-create-widget
			"Set Default Margins"
			(xm-push-button-widget-class)
			form
			(list
			 (list (xm-n-left-attachment) (xm-attach-widget))
			 (list (xm-n-left-widget) rc)
			 (list (xm-n-top-attachment) (xm-attach-widget))
			 (list (xm-n-top-widget) layout-combo)
			 (list (xm-n-right-attachment) (xm-attach-form))
			 (list (xm-n-bottom-attachment) (xm-attach-none)))))

	 (orient-frame (xt-create-widget
			"Page Orientation Frame"
			(xm-frame-widget-class)
			form
			(list
			 (list (xm-n-left-attachment) (xm-attach-widget))
			 (list (xm-n-left-widget) rc)
			 (list (xm-n-top-attachment) (xm-attach-widget))
			 (list (xm-n-top-widget) set-def-marg)
			 (list (xm-n-right-attachment) (xm-attach-form))
			 (list (xm-n-bottom-attachment) (xm-attach-form)))))


	 (paper-or-b (xt-create-widget
		      "Page Orientation Button"
		      (xm-push-button-widget-class)
		      orient-frame
		      (list
		       (list (xm-n-label-type) (xm-pixmap))
		       (list (xm-n-label-pixmap)
			     (if (eqv? (cadr (current-paper-type)) 'portrait)
				 portrait-pix
				 landscape-pix)))))

	 (left-label (form-label rc "Left" #f))
	 (right-label (form-label rc "Right" #f))
	 (top-label (form-label rc "Top" #f))
	 (bottom-label (form-label rc "Bottom" #f))

	 (left-tf (xt-create-widget
		   "Left TF" (xm-text-field-widget-class) rc
		   (list (list (xm-n-columns) 5))))

	 (right-tf (xt-create-widget
		    "Right TF" (xm-text-field-widget-class) rc
		    (list (list (xm-n-columns) 5))))
		    
	 (top-tf (xt-create-widget
		  "Top TF" (xm-text-field-widget-class) rc
		    (list (list (xm-n-columns) 5))))

	 (bottom-tf (xt-create-widget
		     "Bottom TF" (xm-text-field-widget-class) rc
		     (list (list (xm-n-columns) 5))))

	 (layout (container.get-attributes (current-column)))
	 )
    (xt-add-callback dlg (xm-n-ok-callback) page-settings-apply (list #f))
    (xt-add-callback dlg (xm-n-cancel-callback) page-settings-cancel (list #f))

    (xt-manage-child left-label)
    (xt-manage-child right-label)
    (xt-manage-child top-label)
    (xt-manage-child bottom-label)

    (for-each
     (lambda (x) (add-combo-box-item paper-combo (car x)))
     paper-types)
    (let ((pt (current-paper-type)))
      (xm-combo-box-select-position paper-combo (+ 1 (car pt))))
    (xt-manage-child paper-combo)

    (add-combo-box-item layout-combo "Latin")
    (add-combo-box-item layout-combo "Latin 2 Column")
    (add-combo-box-item layout-combo "Asian")
    (add-combo-box-item layout-combo "Arabic")

    (cond ((eqv? layout (car column-styles)) 
	   (xm-combo-box-select-position layout-combo 1))
	  ((eqv? layout (cadr column-styles)) 
	   (xm-combo-box-select-position layout-combo 2))
	  ((eqv? layout (caddr column-styles)) 
	   (xm-combo-box-select-position layout-combo 3))
	  ((eqv? layout (cadddr column-styles)) 
	   (xm-combo-box-select-position layout-combo 4)))

    (xt-manage-child layout-combo)

    (xt-add-callback set-def-marg
		     (xm-n-activate-callback)
		     dlg-set-default-margins
		     (list #f))
		     
    (xt-manage-child set-def-marg)


    (xm-text-set-string
     left-tf (container.get-attribute (current-page) at-margin-low))

    (xm-text-set-string
     right-tf (container.get-attribute (current-page) at-margin-high))

    (xm-text-set-string
     top-tf (container.get-attribute (current-column) at-margin-low))

    (xm-text-set-string
     bottom-tf (container.get-attribute (current-column) at-margin-high))



    (xt-add-callback paper-or-b
		     (xm-n-activate-callback)
		     paper-or-b-callback
		     (list #f))

    (xt-manage-child left-tf)
    (xt-manage-child right-tf)
    (xt-manage-child top-tf)
    (xt-manage-child bottom-tf)

    (xt-manage-child paper-or-b)
    (xt-manage-child orient-frame)

    (xt-manage-child rc)
    (xt-manage-child rc-frame)
    (xt-manage-child form)
    (xt-manage-child frame)
    (xt-manage-child dlg)))


(define (set-widths-by-page-width)
  (let* ((page-size (attributes.get-i (car page-styles) at-fixed-right))
	 (page-m-low (attributes.get-i (car page-styles) at-margin-low))
	 (page-m-high (attributes.get-i (car page-styles) at-margin-high))
	 (page-max (- page-size page-m-low page-m-high)))
    (attributes.add-i (caddr column-styles) at-fixed-left page-max)

    (attributes.add-i (car line-styles-low) at-fixed-right page-max)
    (attributes.add-i (cadr line-styles-low) at-fixed-right (/ page-max 2))
    (attributes.add-i (cadddr line-styles-low) at-fixed-left page-max)

    (attributes.add-i (car line-styles-center) at-fixed-right page-max)
    (attributes.add-i (cadr line-styles-center) at-fixed-right (/ page-max 2))
    (attributes.add-i (cadddr line-styles-center) at-fixed-left page-max)

    (attributes.add-i (car line-styles-high) at-fixed-right page-max)
    (attributes.add-i (cadr line-styles-high) at-fixed-right (/ page-max 2))
    (attributes.add-i (cadddr line-styles-high) at-fixed-left page-max)

    (attributes.add-i (car line-styles-fill) at-fixed-right page-max)
    (attributes.add-i (cadr line-styles-fill) at-fixed-right (/ page-max 2))
    (attributes.add-i (cadddr line-styles-fill) at-fixed-left page-max)
    ))



(define (paper-or-b-callback widget client-data call-data)
  (let* ((cur-pix (xt-get-value widget (xm-n-label-pixmap))))
    (xt-set-value widget (xm-n-label-pixmap)
		  (if (eqv? cur-pix portrait-pix)
		      landscape-pix
		      portrait-pix))))


(define (dlg-set-default-margins widget client-data call-data)
  (let* ((parent (xt-parent widget))

	 (paper-combo (xt-name-to-widget parent "*Paper Combo"))
	 (paper-type-str (xm-combo-box-get-string paper-combo))
	 (paper-type-num (paper-name-to-index paper-type-str))

	 (left-m (list-ref (list-ref paper-types paper-type-num) 3))
	 (top-m (list-ref (list-ref paper-types paper-type-num) 4))
	 (right-m (list-ref (list-ref paper-types paper-type-num) 5))
	 (bottom-m (list-ref (list-ref paper-types paper-type-num) 6))

	 (left-tf (xt-name-to-widget parent "*Left TF"))
	 (top-tf (xt-name-to-widget parent "*Right TF"))
	 (right-tf (xt-name-to-widget parent "*Top TF"))
	 (bottom-tf (xt-name-to-widget parent "*Bottom TF")))
    
    (xm-text-set-string left-tf left-m)
    (xm-text-set-string top-tf top-m)
    (xm-text-set-string right-tf right-m)
    (xm-text-set-string bottom-tf bottom-m) ))


(define (page-settings-apply widget client-data call-data)
  (let* ((left-tf (xt-name-to-widget widget "*Left TF"))
	 (top-tf (xt-name-to-widget widget "*Right TF"))
	 (right-tf (xt-name-to-widget widget "*Top TF"))
	 (bottom-tf (xt-name-to-widget widget "*Bottom TF"))

	 (left-v (* (atoi (xm-text-get-string left-tf)) magnify))
	 (top-v (* (atoi (xm-text-get-string top-tf)) magnify))
	 (right-v (* (atoi (xm-text-get-string right-tf)) magnify))
	 (bottom-v (* (atoi (xm-text-get-string bottom-tf)) magnify))

	 ;(page (current-page))
	 ;(column (current-column))
	 ;(page-attr (container.get-attributes page))
	 ;(column-attr (container.get-attributes column))

	 (paper-combo (xt-name-to-widget widget "*Paper Combo"))
	 (paper-type-str (xm-combo-box-get-string paper-combo))
	 (paper-type-num (paper-name-to-index paper-type-str))
	 (page-width (cadr (list-ref paper-types paper-type-num)))
	 (page-height (caddr (list-ref paper-types paper-type-num)))

	 (layout-combo (xt-name-to-widget widget "*Layout Combo"))
	 (layout-name (xm-combo-box-get-string layout-combo))

	 (cur-or-button (xt-name-to-widget widget "*Page Orientation Button"))
	 (cur-or-pix (xt-get-value cur-or-button (xm-n-label-pixmap)))
	 )


    (set-margins left-v top-v right-v bottom-v)

    (if (eqv? cur-or-pix portrait-pix)
	(set-page-size page-width page-height)
	(set-page-size page-height page-width))


    (cond ((string=? "Latin" layout-name)
	   (set-page #f 0 #f))
	  ((string=? "Latin 2 Column" layout-name)
	   (set-page #f 1 #f))
	  ((string=? "Asian" layout-name)
	   (set-page #f 2 #f))
	  ((string=? "Arabic" layout-name)
	   (set-page #f 3 #f))))

  (xt-destroy-widget widget)
  (xword-window.rewrap xw
		       (container.first-lowest-container doc)
		       (container.last-lowest-container doc))
  (xword-window.scroll-to-cursor xw)
  (xword-window.redraw xw doc))


(define (page-settings-cancel widget client-data call-data)
  (xt-destroy-widget widget))


(define (set-page widget client-data call-data)
  (let* ((col-attr (list-ref column-styles client-data))
	 (line-attr (list-ref line-styles-low client-data))
	 (word-attr (list-ref word-styles client-data))
	 (cursor (xword-window.get-cursor-position xw))
	 (wordsegment (car cursor))
	 (word (container.get-parent wordsegment))
	 (line (container.get-parent word))
	 (column (container.get-parent line))
	 (page (container.get-parent column)))
    (xword-window.set-attributes xw
				 word-attr
				 page
				 (container.find-depth word))
    (xword-window.set-attributes xw
				 line-attr
				 page
				 (container.find-depth line))
    (xword-window.set-attributes xw
				 col-attr
				 page
				 (container.find-depth column))
    (xword-window.rewrap xw
			 (container.first-lowest-container doc)
			 (container.last-lowest-container doc))
    (xword-window.scroll-to-cursor xw)
    (xword-window.redraw xw doc)))




;> I want to know paper sizes.  what is my paper called?  A4?
;> what size is legal paper?  what size do the English use?
;> what other kinds are there?

;Isn't it convenient that a fellow list member works for a company that
;makes printer controllers?

;* Letter: 8.5" x 11".  This is our standard size, the stuff in your laser
;printer.

;* Legal: 8.5" x 14".  The size made famous by those big yellow legal pads.

;* Ledger: 11" x 17".  My mechanical engineers love to use this size to
;print quick proofs of their CAD designs.

;* Executive: 8.25" x 10.5".  I have never known anyone weird enough to
;actually use this.

;* Monarch: 3.875" x 7.5".  Yeah, I don't know either.

;Europe uses paper sizes standardized by the ISO.

;* A3: 297mm x 420mm.  Yes, millimeters.  Damn furriners.  Sorta kinda
;roughly close to ledger.

;* A4: 210mm x 297mm.  Sorta kinda roughly close to letter - Europe's
;standard size, the stuff in their laser printers.

;* A5: 148mm x 210mm.  Don't ask me.

;* B4: 257mm x 364mm.  Used in Japan.

;* B5: 182mm x 257mm.  Japan only.

;* B5: 176mm x 250mm everywhere else.  Yes, that's right, there are two
;different B5 sizes.

;* Com10 envelope: 4.125" x 9.5".  This is a standard full-width business
;size envelope, the kind that conveniently holds a standard letter sheet
;folded in thirds.

;* DL envelope: 110mm x 220mm.  

;* C5 envelope: 162mm x 229mm.

;Certainly, more paper sizes than these exist, but you are unlikely to ever
;encounter them.

